package im.zego.zimchatroomdemo;


import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.Resources;
import android.os.Build;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;

import im.zego.zimchatroomdemo.utils.KeyCenter;
import im.zego.zimchatroom.ZIMChatRoom;
import im.zego.zimchatroom.callback.CreateChatRoomCallback;
import im.zego.zimchatroom.callback.JoinChatRoomCallback;
import im.zego.zimchatroom.emus.ZIMChatRoomErrorCode;
import im.zego.zimchatroom.internal.entity.ZIMChatRoomInfo;
import im.zego.zimchatroom.util.TokenServerAssistant;
import im.zego.zimchatroom.util.ZegoRTCServerAssistant;
import im.zego.zimchatroomdemo.roomchatActivity.OwnerRoomActivity;
import im.zego.zimchatroomdemo.roomchatActivity.VisitorRoomActivity;
import im.zego.zimchatroomdemo.utils.MyToast;

import org.json.JSONException;

public class RoomLoginActivity extends MyBaseActivity {
    private EditText etRoomID;
    private EditText etRoomName;
    private Button btnCreate;
    private Button btnJoin;
    public static boolean isTouched;
    //String info for login room
    public static String roomName;
    public static String roomID;
    //上层获取的数据
    String userID, userName;
    //获取ChatRoomSDK
    ZIMChatRoom zimChatRoom = ZIMChatRoom.getInstance();
    //roomInfo设置
    ZIMChatRoomInfo chatRoomInfo;
    //获取国际化资源
    Resources resources;

    // 最佳启用该Activity的方法
    public static void startActivity(Activity activity, String userID, String userName) {
        Intent intent = new Intent(activity, RoomLoginActivity.class);
        intent.putExtra("userID", userID);
        intent.putExtra("userName", userName);
        activity.startActivity(intent);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_room_login);
        userID = getIntent().getStringExtra("userID");
        userName = getIntent().getStringExtra("userName");
        resources = getResources();
        initUI();
    }

    @Override
    protected void onResume() {
        super.onResume();
        /*chatRoomInfo = RoomBaseActivity.chatRoomInfo;
        //房间属性默认为8麦位，不禁言,不锁麦
        chatRoomInfo.setSeat_count(8);
        chatRoomInfo.setIs_mute_msg(false);
        chatRoomInfo.setLocked_seat_list(null);*/
    }

    private static String[] PERMISSIONS_STORAGE = {
            "android.permission.RECORD_AUDIO"};

    public boolean checkOrRequestPermission(int code) {
        System.out.println("SDK_INT = " + Build.VERSION.SDK_INT + ", M = " + Build.VERSION_CODES.M);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (ContextCompat.checkSelfPermission(this, "android.permission.RECORD_AUDIO") != PackageManager.PERMISSION_GRANTED) {
                requestPermissions(PERMISSIONS_STORAGE, code);
                return false;
            }
        }
        return true;
    }

    private static final int REQUEST_PERMISSION_CODE = 101;

    private void initUI() {
        etRoomID = findViewById(R.id.et_roomID);
        etRoomName = findViewById(R.id.et_roomName);
        btnCreate = findViewById(R.id.btn_create);
        btnJoin = findViewById(R.id.btn_join);
        btnJoin.setOnClickListener(this);
        btnCreate.setOnClickListener(this);

        //限制房间名称的长度
        etRoomName.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                if (s.length() > 8) { //判断EditText中输入的字符数是不是已经大于8,八个汉字
                    etRoomName.setText(s.toString().substring(0, 8)); //设置EditText只显示前面8位字符
                    etRoomName.setSelection(8);//让光标移至末端
                    MyToast.showShortTimeToast(RoomLoginActivity.this, resources.getString(R.string.toast_chatroom_name_too_long));
                }
                return;
            }

            @Override
            public void afterTextChanged(Editable s) {
            }
        });
    }

    public void back(View view) {
        finish();
    }

    @Override
    protected void onDestroy() {
        zimChatRoom.logout();
        super.onDestroy();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == 100) {
            finish();
        }
    }

    @Override
    public void onClick(View v) {
        boolean orRequestPermission = checkOrRequestPermission(REQUEST_PERMISSION_CODE);
        if (orRequestPermission) {
            roomName = etRoomName.getText().toString();
            roomID = etRoomID.getText().toString();
            ZegoRTCServerAssistant.Privileges privileges = new ZegoRTCServerAssistant.Privileges();
            privileges.canLoginRoom = true;
            privileges.canPublishStream = true;


            //构建房间信息
            switch (v.getId()) {
                case R.id.btn_create:
                    if (TextUtils.isEmpty(roomID)) {
                        MyToast.showShortTimeToast(RoomLoginActivity.this, resources.getString(R.string.toast_roomid_enter_fail));
                        return;
                    }

                    if (roomID.length() > 32) {
                        MyToast.showShortTimeToast(RoomLoginActivity.this, "房间ID不能超过32字符，创建房间失败!");
                        return;
                    }
                    if (TextUtils.isEmpty(roomName)) {
                        MyToast.showShortTimeToast(RoomLoginActivity.this, resources.getString(R.string.toast_chatroom_name_enter_fail));
                        roomName = roomID;
                        return;
                    }
                    if ((!TextUtils.isEmpty(roomID)) && (!TextUtils.isEmpty(roomName))) {
                        zimChatRoom.createChatRoom(roomID, roomName, ZegoRTCServerAssistant.generateToken(KeyCenter.appID(), roomID, userID, privileges, KeyCenter.appExpressSign(), 660).data, new CreateChatRoomCallback() {
                            @Override
                            public void onCreateRoomState(ZIMChatRoomErrorCode error) {
                                if (error == ZIMChatRoomErrorCode.SUCCESS) {
                                    OwnerRoomActivity.actionStart(RoomLoginActivity.this, roomID, roomName, userID, userName);
                                } else if (error == ZIMChatRoomErrorCode.ROOM_EXISTED) {
                                    MyToast.showShortTimeToast(RoomLoginActivity.this, resources.getString(R.string.toast_room_existed));
                                }
                            }
                        });
                    }
                    break;

                case R.id.btn_join:
                    if (roomID.length() > 32) {
                        MyToast.showShortTimeToast(RoomLoginActivity.this, "房间ID不能超过32字符，创建房间失败!");
                        return;
                    }
                    if (TextUtils.isEmpty(roomID)) {
                        MyToast.showShortTimeToast(RoomLoginActivity.this, resources.getString(R.string.toast_roomid_enter_fail));

                    } else {

                        try {
                            zimChatRoom.joinChatRoom(roomID, TokenServerAssistant.generateToken(KeyCenter.appID(), userID, KeyCenter.appZIMServerSecret(), 660).data, new JoinChatRoomCallback() {
                                @Override
                                public void onConnectionState(ZIMChatRoomErrorCode error) {
                                    if (error == ZIMChatRoomErrorCode.SUCCESS) {
                                        isTouched = true;
                                        //进行加入房间操作
                                        VisitorRoomActivity.actionStart(RoomLoginActivity.this, roomID, roomName, userID, userName);
                                    } else if (error == ZIMChatRoomErrorCode.ROOM_NOT_FOUND) {
                                        MyToast.showShortTimeToast(RoomLoginActivity.this, resources.getString(R.string.toast_join_room_fail));
                                    }
                                }
                            });
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                        break;
                    }

            }
        }
    }
}