package im.zego.zimchatroomdemo.adapter;

import static im.zego.zimchatroom.emus.ZIMChatRoomVoiceStatus.LOCKED;
import static im.zego.zimchatroom.emus.ZIMChatRoomVoiceStatus.UNUSED;
import static im.zego.zimchatroom.emus.ZIMChatRoomVoiceStatus.USED;

import android.content.res.Resources;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import im.zego.zimchatroom.entity.ZIMVoiceSeat;
import im.zego.zimchatroomdemo.R;
import im.zego.zimchatroomdemo.adapter.holder.UserInChatListHolder;
import im.zego.zimchatroomdemo.enums.RoomType;
import im.zego.zimchatroomdemo.roomchatActivity.RoomBaseActivity;

import java.util.ArrayList;

//展示所有在舞台中麦位用户列表，包括上麦和闭麦等等
public class UserInChatListAdapter extends RecyclerView.Adapter {
    public ArrayList<ZIMVoiceSeat> getSeatListInRoom() {
        return seatList;
    }

    //相关的Seat参数等到后期再进行设计
    private ArrayList<ZIMVoiceSeat> seatList = new ArrayList<>();
    RoomType roomType;
    Resources resources;

    public void setAuthorID(String authorID) {
        this.authorID = authorID;
    }

    String authorID;

    public UserInChatListAdapter(RoomType roomType) {
        this.seatList = initAllSeatListInRoom();
        this.roomType = roomType;

        resources = RoomBaseActivity.resources;
    }

    public ChatListItemOnClick click = null;

    /*

     */
    private static ArrayList<ZIMVoiceSeat> initAllSeatListInRoom() {
        ArrayList<ZIMVoiceSeat> allSeats = new ArrayList<>();
        //初始化所有麦位信息，数组下标即为麦位index
        for (int i = 0; i <= 7; i++) {
            ZIMVoiceSeat voiceSeat = new ZIMVoiceSeat();
            voiceSeat.getAttribution().setIndex(i);
            allSeats.add(voiceSeat);
        }
        return allSeats;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        //视图绑定，通过获取它的view进行渲染
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_chat_user, parent, false);
        UserInChatListHolder holder = new UserInChatListHolder(view);
        return holder;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        UserInChatListHolder userInChatListHolder = (UserInChatListHolder) holder;

        ZIMVoiceSeat voiceSeat = seatList.get(position);

        // 开始使用该麦位
        userInChatListHolder.tvUserName.setText(voiceSeat.getAttribution().getUser_id());
        userInChatListHolder.relativeLayout.setTag(position);
        userInChatListHolder.relativeLayout.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int index = (int) v.getTag();
                final ChatListItemOnClick chatListItemOnClick = click;
                if (chatListItemOnClick != null) {
                    chatListItemOnClick.onClick(index);
                }
            }
        });

        if (authorID != null && authorID.equals(voiceSeat.getAttribution().getUser_id())) {
            userInChatListHolder.ivOwnerAvatar.setVisibility(View.VISIBLE);
        } else {
            userInChatListHolder.ivOwnerAvatar.setVisibility(View.GONE);
        }

        userInChatListHolder.ivAvatarTalking.setBackgroundColor(Color.WHITE);
        if (voiceSeat.getStatus() == USED) {
            userInChatListHolder.progress.setVisibility(View.GONE);
            userInChatListHolder.ivAvatar.setVisibility(View.VISIBLE);
            if (voiceSeat.getAttribution().isIs_muted()) {
                userInChatListHolder.ivMicOff.setVisibility(View.VISIBLE);
                userInChatListHolder.ivMicOn.setVisibility(View.GONE);
            } else {
                userInChatListHolder.ivMicOff.setVisibility(View.GONE);
                userInChatListHolder.ivMicOn.setVisibility(View.VISIBLE);
            }
        }
//        } else if (voiceSeat.getStatus() == USING) {
//            userInChatListHolder.progress.setVisibility(View.GONE);
//            userInChatListHolder.ivAvatar.setVisibility(View.VISIBLE);
//            if (voiceSeat.getAttribution().isMuted()) {
//                userInChatListHolder.ivMicOff.setVisibility(View.VISIBLE);
//                userInChatListHolder.ivMicOn.setVisibility(View.GONE);
//            } else {
//                userInChatListHolder.ivMicOff.setVisibility(View.GONE);
//                userInChatListHolder.ivMicOn.setVisibility(View.VISIBLE);
//            }
//
        else if (voiceSeat.getStatus() == UNUSED) {
            userInChatListHolder.progress.setVisibility(View.GONE);
            userInChatListHolder.ivAvatar.setVisibility(View.GONE);
            userInChatListHolder.ivMicOff.setVisibility(View.GONE);
            userInChatListHolder.ivMicOn.setVisibility(View.GONE);
            userInChatListHolder.tvUserName.setText("");
        } else if (voiceSeat.getStatus() == LOCKED) {
            userInChatListHolder.ivAvatarTalking.setBackgroundColor(Color.GRAY);
            userInChatListHolder.progress.setVisibility(View.GONE);
            userInChatListHolder.tvUserName.setText("");
            userInChatListHolder.ivAvatar.setVisibility(View.GONE);
            userInChatListHolder.ivMicOff.setVisibility(View.GONE);
            userInChatListHolder.ivMicOn.setVisibility(View.GONE);
        }
    }

    @Override
    public int getItemCount() {
        return seatList.size();//默认为8个麦位
    }


    public void updateUserInfo(int index, ZIMVoiceSeat zimVoiceSeat) {
        seatList.set(index, zimVoiceSeat);
        this.notifyItemChanged(index);

    }

    public void setItemOnClick(ChatListItemOnClick click) {
        this.click = click;
    }

    public interface ChatListItemOnClick {
        public void onClick(int index);
    }
}
