package im.zego.zimchatroomdemo.roomchatActivity;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Toast;

import im.zego.zimchatroom.callback.EnterSeatCallback;
import im.zego.zimchatroom.callback.KickUserToSeatCallback;
import im.zego.zimchatroom.callback.LockSeatCallback;
import im.zego.zimchatroom.emus.ZIMChatRoomErrorCode;
import im.zego.zimchatroom.emus.ZIMChatRoomVoiceStatus;
import im.zego.zimchatroom.entity.ZIMChatRoomUser;
import im.zego.zimchatroom.entity.ZIMVoiceSeat;
import im.zego.zimchatroomdemo.R;
import im.zego.zimchatroomdemo.adapter.UserInChatListAdapter;
import im.zego.zimchatroomdemo.enums.RoomType;

public class OwnerRoomActivity extends RoomBaseActivity {
    //启动一个Activity的最佳写法
    public static void actionStart(Activity activity, String roomID, String roomName, String userID, String userName) {
        Intent intent = new Intent(activity, OwnerRoomActivity.class);
        intent.putExtra("roomID", roomID);
        intent.putExtra("roomName", roomName);
        intent.putExtra("userID", userID);
        intent.putExtra("userName", userName);
        activity.startActivityForResult(intent, 100);//进行activity的跳转
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        roomID = getIntent().getStringExtra("roomID");
        roomName = getIntent().getStringExtra("roomName");
        userID = getIntent().getStringExtra("userID");
        userName = getIntent().getStringExtra("userName");
        user = new ZIMChatRoomUser();
        user.setUserID(userID);
        user.setUserName(userName);
        user.setUserRole(userRole);
        chatRoomInfo.setRoom_id(roomID);
        chatRoomInfo.setRoom_Name(roomName);
        chatRoomInfo.setAuthor(userID);

        //房主上麦
        chatRoom.enterSeat(0, new EnterSeatCallback() {
            @Override
            public void enterSeat(ZIMChatRoomErrorCode error) {

            }
        });

        userInChatListAdapter.setAuthorID(userID);

        // 设置麦位管理点击事件
        userInChatListAdapter.setItemOnClick(new UserInChatListAdapter.ChatListItemOnClick() {
            @Override
            public void onClick(final int index) {
                final ZIMVoiceSeat seat = userInChatListAdapter.getSeatListInRoom().get(index);
                if (seat.getStatus() == ZIMChatRoomVoiceStatus.UNUSED) {
                    userInChatListAdapter.updateUserInfo(index, seat);
                    chatRoom.lockSeat(true, index, new LockSeatCallback() {
                        @Override
                        public void lockSeat(ZIMChatRoomErrorCode error) {
                            if (error == ZIMChatRoomErrorCode.SUCCESS) {
                                seat.setStatus(ZIMChatRoomVoiceStatus.LOCKED);
                                userInChatListAdapter.updateUserInfo(index, seat);
                            }
                        }
                    });
                } else if (seat.getStatus() == ZIMChatRoomVoiceStatus.LOCKED) {
                    userInChatListAdapter.updateUserInfo(index, seat);
                    chatRoom.lockSeat(false, index, new LockSeatCallback() {
                        @Override
                        public void lockSeat(ZIMChatRoomErrorCode error) {
                            if (error == ZIMChatRoomErrorCode.SUCCESS) {
                                seat.setStatus(ZIMChatRoomVoiceStatus.UNUSED);
                                userInChatListAdapter.updateUserInfo(index, seat);
                            }
                        }
                    });
                } else if (seat.getStatus() == ZIMChatRoomVoiceStatus.USED) {
                    final String toUserID = userInChatListAdapter.getSeatListInRoom().get(index).getAttribution().getUser_id();
                    if (!userID.equals(toUserID)) {
                        AlertDialog.Builder builder = new AlertDialog.Builder(OwnerRoomActivity.this);
                        builder.setTitle(getResources().getString(R.string.dialog_warning));
                        builder.setMessage(getResources().getString(R.string.dialog_warning_message, toUserID));
                        builder.setNegativeButton(getResources().getString(R.string.dialog_confirm), new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                chatRoom.kickUserToSeat(toUserID, new KickUserToSeatCallback() {
                                    @Override
                                    public void kickUserToSeat(ZIMChatRoomErrorCode error) {
                                        if (error == ZIMChatRoomErrorCode.NOT_IN_SEAT) {
                                            Toast.makeText(OwnerRoomActivity.this, "用户已不在麦上", Toast.LENGTH_LONG).show();
                                        }
                                    }
                                });
                            }
                        });

                        builder.setPositiveButton(getResources().getString(R.string.dialog_cancel), new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.cancel();
                            }
                        });

                        AlertDialog dialog = builder.create();
                        dialog.show();
                    }
                }
            }
        });


    }

    @Override
    protected void onResume() {
        super.onResume();

    }


    public boolean onKeyDown(int keyCode, KeyEvent event) {

        if (keyCode == KeyEvent.KEYCODE_BACK) {
            showDialog();
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    public void showDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(OwnerRoomActivity.this);
        builder.setTitle(getResources().getString(R.string.dialog_warning));
        builder.setMessage("是否解散该房间？");
        builder.setNegativeButton("解散", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                finish();
                dialog.cancel();
            }
        });
        builder.setPositiveButton("取消", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {

                dialog.cancel();
            }
        });
        AlertDialog dialog = builder.create();
        dialog.show();

    }

    @Override
    protected void initUI() {
        super.initUI();
    }

    @Override
    protected RoomType getRoomType() {
        return RoomType.TYPE_ROOM_OWNER;
    }

    @Override
    public void onClick(View v) {

        switch (v.getId()) {
            case R.id.iv_back:
                showDialog();
                return;
        }
        super.onClick(v);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

}
