package im.zego.zimchatroom;

import android.app.Application;

import im.zego.zimchatroom.callback.CreateChatRoomCallback;
import im.zego.zimchatroom.callback.EnterSeatCallback;
import im.zego.zimchatroom.callback.JoinChatRoomCallback;
import im.zego.zimchatroom.callback.KickUserToSeatCallback;
import im.zego.zimchatroom.callback.LeaveSeatCallback;
import im.zego.zimchatroom.callback.LockSeatCallback;
import im.zego.zimchatroom.callback.MuteAllMessageCallback;
import im.zego.zimchatroom.callback.MuteSeatCallback;
import im.zego.zimchatroom.callback.OnLeaveCallback;
import im.zego.zimchatroom.callback.OnLoginCallback;
import im.zego.zimchatroom.callback.OnQueryChatRoomInfoCallback;
import im.zego.zimchatroom.callback.OnQueryRoomMemberCallback;
import im.zego.zimchatroom.callback.SendGiftMessageCallback;
import im.zego.zimchatroom.callback.SendInvitationStatusCallback;
import im.zego.zimchatroom.callback.SendRoomMessageCallback;
import im.zego.zimchatroom.callback.SwitchSeatCallback;
import im.zego.zimchatroom.callback.ZIMChatRoomEventHandler;
import im.zego.zimchatroom.emus.ZIMChatRoomInvitationStatus;
import im.zego.zimchatroom.entity.ZIMChatRoomQueryMemberConfig;
import im.zego.zimchatroom.entity.ZIMChatRoomUserInfo;
import im.zego.zimchatroom.internal.impl.ZIMChatRoomImpl;

import java.util.ArrayList;

public abstract class ZIMChatRoom {
    public ZIMChatRoom() {
    }

    //单例实现
    public static ZIMChatRoom getInstance() {
        return ZIMChatRoomImpl.getInstance();
    }

    // handler 设置
    public abstract void setZIMChatRoomEventHandler(ZIMChatRoomEventHandler chatRoomEventHandler);

    // 初始化
    public abstract void init(Long appID, String appSign, Application application);

    // 登录服务
    public abstract void login(ZIMChatRoomUserInfo userInfo, String token, OnLoginCallback onLoginCallback);

    // 更新 RTC Token
    public abstract void renewRTCToken(String token);

    // 更新 ZIM Token
    public abstract void renewZIMToken(String token);

    // 登出
    public abstract void logout();

    // 反初始化
    public abstract void unInit();

    /**
     * 房间模块
     */
    // 创建语聊房
    public abstract void createChatRoom(String roomID, String roomName, String rtcToken, CreateChatRoomCallback createChatRoomCallback);

    // 加入语聊房
    public abstract void joinChatRoom(String roomID, String rtcToken, JoinChatRoomCallback joinChatRoomCallback);

    // 离开语聊房
    public abstract void leaveChatRoom(String roomID, OnLeaveCallback onLeaveCallback);

    //分页拉取房间用户
    public abstract void queryRoomMember(String roomID, ZIMChatRoomQueryMemberConfig config, OnQueryRoomMemberCallback onQueryRoomMember);

    //主动获取房间信息
    public abstract void queryChatRoomInfo(String roomID, OnQueryChatRoomInfoCallback onQueryRoomMember);

    /**
     * 消息模块
     */
    //发送房间消息
    public abstract void sendRoomMessage(String message, SendRoomMessageCallback sendRoomMessageCallback);

    //发送礼物消息
    public abstract void sendGiftMessage(int giftType, ArrayList<String> userIDs, SendGiftMessageCallback sendGiftMessageCallback);

    //发送邀请 判断是否发送成功
    public abstract void sendInvitation(String toUserID, SendInvitationStatusCallback sendInvitationStatusCallback);

    //回应邀请
    public abstract void respondInvitation(ZIMChatRoomInvitationStatus status, SendInvitationStatusCallback sendInvitationStatusCallback);

    //全员禁言的信令同步，开发者收到回调之后需要自行在 UI 端实现诸如输入框不能点击等操作
    public abstract void muteAllMessage(boolean isMuted, MuteAllMessageCallback muteAllMessageCallback);


    /**
     * 麦位模块
     */
    //踢观众下麦
    public abstract void kickUserToSeat(String userID, KickUserToSeatCallback kickUserToSeat);

    //锁麦和解锁麦位
    public abstract void lockSeat(boolean isLocked, int seatIndex, LockSeatCallback lockSeat);

    //静音（麦克风）该用户,这个是房主和观众都可用的（仅限在上麦）
    public abstract void muteSeat(boolean isMuted, MuteSeatCallback muteSeat);

    //上麦
    public abstract void enterSeat(int seatIndex, EnterSeatCallback enterSeatCallback);

    //下麦
    public abstract void leaveSeat(LeaveSeatCallback leaveSeatCallback);

    //跳麦
    public abstract void switchSeat(int toSeatIndex, SwitchSeatCallback switchSeatCallback);

    /**
     * 设备模块
     */
    // 开启和关闭扬声器
    public abstract void muteSpeaker(boolean isMuted);

}