package im.zego.zimchatroom.internal;

import static im.zego.zimchatroom.emus.ZIMChatRoomErrorCode.SUCCESS;

import android.app.Application;

import im.zego.zim.callback.ZIMRoomAttributesQueriedCallback;
import im.zego.zim.callback.ZIMRoomMemberQueriedCallback;
import im.zego.zim.entity.ZIMRoomMemberQueryConfig;
import im.zego.zimchatroom.callback.CreateChatRoomCallback;
import im.zego.zimchatroom.callback.EnterSeatCallback;
import im.zego.zimchatroom.callback.JoinChatRoomCallback;
import im.zego.zimchatroom.callback.KickUserToSeatCallback;
import im.zego.zimchatroom.callback.LeaveSeatCallback;
import im.zego.zimchatroom.callback.LockSeatCallback;
import im.zego.zimchatroom.callback.MuteAllMessageCallback;
import im.zego.zimchatroom.callback.MuteSeatCallback;
import im.zego.zimchatroom.callback.OnLeaveCallback;
import im.zego.zimchatroom.callback.OnLoginCallback;
import im.zego.zimchatroom.callback.OnQueryChatRoomInfoCallback;
import im.zego.zimchatroom.callback.OnQueryRoomMemberCallback;
import im.zego.zimchatroom.callback.SendGiftMessageCallback;
import im.zego.zimchatroom.callback.SendInvitationStatusCallback;
import im.zego.zimchatroom.callback.SendRoomMessageCallback;
import im.zego.zimchatroom.callback.SetupRTCModuleCallback;
import im.zego.zimchatroom.callback.SwitchSeatCallback;
import im.zego.zimchatroom.callback.ZIMChatRoomEventHandler;
import im.zego.zimchatroom.emus.ZIMChatRoomErrorCode;
import im.zego.zimchatroom.emus.ZIMChatRoomEvent;
import im.zego.zimchatroom.emus.ZIMChatRoomInvitationStatus;
import im.zego.zimchatroom.emus.ZIMChatRoomState;
import im.zego.zimchatroom.emus.ZIMChatRoomUserRole;
import im.zego.zimchatroom.emus.ZIMChatRoomVoiceStatus;
import im.zego.zimchatroom.entity.ZIMChatRoomGift;
import im.zego.zimchatroom.entity.ZIMChatRoomGiftBroadcast;
import im.zego.zimchatroom.internal.entity.ZIMChatRoomInfo;
import im.zego.zimchatroom.entity.ZIMChatRoomInvitationRespond;
import im.zego.zimchatroom.entity.ZIMChatRoomMessage;
import im.zego.zimchatroom.entity.ZIMChatRoomQueryMemberConfig;
import im.zego.zimchatroom.entity.ZIMChatRoomUserInfo;
import im.zego.zimchatroom.entity.ZIMVoiceSeat;
import im.zego.zimchatroom.entity.ZIMChatRoomSeatAttribution;
import im.zego.zimchatroom.entity.ZIMChatRoomText;
import im.zego.zimchatroom.entity.ZIMChatRoomUser;
import im.zego.zimchatroom.entity.ZIMVoiceSeatUpdateInfo;
import im.zego.zimchatroom.util.GsonChanger;

import com.google.gson.JsonSyntaxException;

import org.json.JSONObject;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.constants.ZegoScenario;
import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMEventHandler;
import im.zego.zim.callback.ZIMLoggedInCallback;
import im.zego.zim.callback.ZIMRoomCreatedCallback;
import im.zego.zim.callback.ZIMRoomJoinedCallback;
import im.zego.zim.callback.ZIMRoomLeftCallback;
import im.zego.zim.entity.ZIMError;
import im.zego.zim.entity.ZIMMessage;
import im.zego.zim.entity.ZIMRoomAdvancedConfig;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;
import im.zego.zim.entity.ZIMRoomFullInfo;
import im.zego.zim.entity.ZIMRoomInfo;
import im.zego.zim.entity.ZIMTextMessage;
import im.zego.zim.entity.ZIMUserInfo;
import im.zego.zim.enums.ZIMConnectionEvent;
import im.zego.zim.enums.ZIMConnectionState;
import im.zego.zim.enums.ZIMErrorCode;
import im.zego.zim.enums.ZIMRoomAttributesUpdateAction;
import im.zego.zim.enums.ZIMRoomEvent;
import im.zego.zim.enums.ZIMRoomState;


//Manager都是具体实现逻辑的地方
public class ZIMChatRoomManager {
    ZegoExpressEngine zegoExpressEngine;
    ZIM zim;
    ZIMChatRoomUser mUser = new ZIMChatRoomUser();
    Application application;
    ZIMEventHandler zimEventHandler;//这个是唯一的，容易被覆盖，所以要进行传递

    //本SDK 自带的参数
    ZIMChatRoomInfo mChatRoomInfo;
    //消息管理者
    ZIMMessageManager messageManager;
    //麦位管理者
    ZIMVoiceSeatManager voiceSeatManager;
    //eventHandler
    ZIMChatRoomEventHandler zimChatRoomEventHandler;
    static ZIMChatRoomManager instance;

    public static ZIMChatRoomManager getInstance() {
        if (instance == null) {
            instance = new ZIMChatRoomManager();
        }
        return instance;
    }

    public ZIMChatRoomManager() {
        messageManager = new ZIMMessageManager();
        voiceSeatManager = new ZIMVoiceSeatManager();
    }

    // 设置handler
    public void setEventHandler(ZIMChatRoomEventHandler chatRoomEventHandler) {
        this.zimChatRoomEventHandler = chatRoomEventHandler;
    }

    public ZIMChatRoomEventHandler getEventHandler() {
        return this.zimChatRoomEventHandler;
    }

    //ZIMHandler的初始化为了实现
    private void initZIMEventHandler() {
        zimEventHandler = new ZIMEventHandler() {
            /**
             消息管理中的回调参数
             */
            @Override
            public void onReceivePeerMessage(ZIM zim, ArrayList<ZIMMessage> messageList, String fromUserID) {
                super.onReceivePeerMessage(zim, messageList, fromUserID);
                for (ZIMMessage zimMessage : messageList) {
                    ZIMTextMessage message = (ZIMTextMessage) zimMessage;
                    ZIMChatRoomMessage chatRoomMessage = GsonChanger.getInstance().getZIMChatRoomMessage(message.message);
                    final ZIMChatRoomEventHandler chatRoomEventHandler = zimChatRoomEventHandler;
                    switch (chatRoomMessage.getAction()) {
                        case INVITATION:
                            //意味着这个只是一个简单的邀请信息，不是要进行反馈的信息；
                            if (chatRoomEventHandler != null) {
                                chatRoomEventHandler.onReceiveInvitation(fromUserID);//增加到handler中
                            }
                            messageManager.fromInvitationUserID = fromUserID;

                            break;
                        case INVITATION_RESPOND:
                            ZIMChatRoomInvitationRespond invitationRespond = chatRoomMessage.getInvitationRespond();
                            //发送过来的是一个反馈的邀请回执,回调方法
                            if (chatRoomEventHandler != null) {
                                chatRoomEventHandler.onResponseInvitation(invitationRespond.getStatus(), fromUserID);
                            }
                            break;
                        case GIFT:
                            //在接收到礼物消息的时候要进行解析，谁送的，送给谁
                            ZIMChatRoomGift gift = chatRoomMessage.getGiftMessage();
                            if (chatRoomEventHandler != null) {
                                chatRoomEventHandler.onReceiveGiftMessage(gift.getGiftType(), gift.getFromUserID());
                            }
                            break;
                    }
                }
            }

            @Override
            public void onReceiveRoomMessage(ZIM zim, ArrayList<ZIMMessage> messageList, String fromRoomID) {
                super.onReceiveRoomMessage(zim, messageList, fromRoomID);
                final ZIMChatRoomEventHandler chatRoomEventHandler = zimChatRoomEventHandler;
                //这里对应的是消息管理中接收房间消息和接收礼物广播的具体回调
                for (ZIMMessage zimMessage : messageList) {
                    ZIMTextMessage message = (ZIMTextMessage) zimMessage;
                    ZIMChatRoomMessage chatRoomMessage = GsonChanger.getInstance().getZIMChatRoomMessage(message.message);
                    switch (chatRoomMessage.getAction()) {
                        case GIFT_BROADCAST:
                            ZIMChatRoomGiftBroadcast giftBroadcast = chatRoomMessage.getGiftBroadcast();
                            if (chatRoomEventHandler != null) {
                                chatRoomEventHandler.onReceiveGiftBroadcastMessage(giftBroadcast.getToUserIDList(), giftBroadcast.getGiftType(), zimMessage.getSenderUserID());
                            }
                            break;
                        case TEXT:
                            ZIMChatRoomText text = chatRoomMessage.getTextMessage();
                            if (chatRoomEventHandler != null) {
                                chatRoomEventHandler.onReceiveRoomMassage(text.getContent(), zimMessage.getSenderUserID());
                            }
                    }
                }
            }

            @Override
            public void onRoomStateChanged(ZIM zim, ZIMRoomState state, ZIMRoomEvent event, JSONObject extendedData, String roomID) {
                super.onRoomStateChanged(zim, state, event, extendedData, roomID);
                final ZIMChatRoomEventHandler chatRoomEventHandler = zimChatRoomEventHandler;
                ZIMChatRoomState chatRoomState = ZIMChatRoomState.getZIMChatRoomState(state.value());
                ZIMChatRoomEvent chatRoomEvent = ZIMChatRoomEvent.getZIMChatRoomEvent(event.value());
                if (chatRoomEventHandler != null) {
                    chatRoomEventHandler.onChatRoomStateUpdated(chatRoomState, chatRoomEvent, roomID);
                }
            }

            @Override
            public void onTokenWillExpire(ZIM zim, int second) {
                super.onTokenWillExpire(zim, second);
                final ZIMChatRoomEventHandler chatRoomEventHandler = zimChatRoomEventHandler;
                //这里进行回调弹出Token要过期的东西
                if (chatRoomEventHandler != null) {
                    chatRoomEventHandler.onZIMTokenWillExpire(second);
                }
            }

            /**
             用户模块的回调
             */
            @Override
            public void onRoomMemberJoined(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                super.onRoomMemberJoined(zim, memberList, roomID);
                ArrayList<ZIMChatRoomUser> chatRoomUsers = getChatRoomUsers(memberList);//转化
                final ZIMChatRoomEventHandler chatRoomEventHandler = zimChatRoomEventHandler;
                if (chatRoomEventHandler != null) {
                    chatRoomEventHandler.onChatRoomMemberJoined(chatRoomUsers);
                }
            }

            @Override
            public void onRoomMemberLeft(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                super.onRoomMemberLeft(zim, memberList, roomID);
                final ZIMChatRoomEventHandler chatRoomEventHandler = zimChatRoomEventHandler;
                ArrayList<ZIMChatRoomUser> chatRoomUsers = getChatRoomUsers(memberList);//转化
                if (chatRoomEventHandler != null) {
                    chatRoomEventHandler.onChatRoomMemberLeft(chatRoomUsers);
                }
            }

            /**
             麦位变化监听回调（ZIM房间属性）
             */
            @Override
            public void onRoomAttributesUpdated(ZIM zim, ZIMRoomAttributesUpdateInfo info, String roomID) {
                super.onRoomAttributesUpdated(zim, info, roomID);
                final ZIMChatRoomEventHandler chatRoomEventHandler = zimChatRoomEventHandler;
                ArrayList<ZIMVoiceSeatUpdateInfo> seatUpdateInfos;
                //判断是增加就存储起来，delete就删除其中的消息
                Set<String> keys = info.roomAttributes.keySet();
                for (String key : keys) {
                    //非房间属性的变更
                    if (!key.equals("room_info")) {
                        if (info.action == ZIMRoomAttributesUpdateAction.SET) {
                            ZIMChatRoomSeatAttribution seatAttribution = null;
                            try {
                                seatAttribution = GsonChanger.getInstance().getZIMChatRoomSeatAttribution(info.roomAttributes.get(key));
                            } catch (JsonSyntaxException exception) {
                                exception.printStackTrace();
                            }
                            if (seatAttribution == null) {
                                continue;
                            }
                            // 有 user 上麦或锁麦
                            if (!voiceSeatManager.isVoiceSeatUsed(seatAttribution.getIndex())) {
                                if (seatAttribution.isIs_locked()) {
                                    voiceSeatManager.mVideoSeats.get(seatAttribution.getIndex()).getAttribution().setUser_id("");
                                    voiceSeatManager.onVoiceSeatLocked(seatAttribution.getIndex(), true);
                                } else {
                                    ZIMVoiceSeat chatRoomSeat = new ZIMVoiceSeat();
                                    chatRoomSeat.setAttribution(seatAttribution);
                                    chatRoomSeat.setStatus(ZIMChatRoomVoiceStatus.USED);
                                    voiceSeatManager.onVoiceSeatAdded(chatRoomSeat);
                                }
                            }
                            // 有 user 更改了麦位里的信息，当前只有静麦可实时更改
                            else {
                                // ZIMVoiceSeat currentSeat = voiceSeatManager.getVoiceSeat(seatAttribution.getIndex());
                                voiceSeatManager.onVoiceSeatMuted(seatAttribution.getIndex(), seatAttribution.isIs_muted());
                            }
                        } else if (info.action == ZIMRoomAttributesUpdateAction.DELETE) {
                            String idxStr = key.split("_")[1];
                            int index = Integer.parseInt(idxStr);
                            ZIMChatRoomSeatAttribution seatAttribution = null;
                            try {
                                seatAttribution = GsonChanger.getInstance().getZIMChatRoomSeatAttribution(info.roomAttributes.get(key));
                            } catch (JsonSyntaxException exception) {
                                exception.printStackTrace();
                            }
                            ZIMVoiceSeat oldSeat = voiceSeatManager.getVoiceSeat(index);
                            if (oldSeat.getStatus() == ZIMChatRoomVoiceStatus.LOCKED) {
                                voiceSeatManager.onVoiceSeatLocked(index, false);
                            } else if (oldSeat.getStatus() == ZIMChatRoomVoiceStatus.USED) {
                                voiceSeatManager.onVoiceSeatRemoved(index);
                            }
                            if (seatAttribution == null) {
                                continue;
                            }
                            voiceSeatManager.mVideoSeats.get(seatAttribution.getIndex()).getAttribution().setUser_id("");

                        }
                    } else {
                        //更改房间属性
                        if (info.action == ZIMRoomAttributesUpdateAction.SET) {
                            ZIMChatRoomInfo roomInfo = GsonChanger.getInstance().getZIMChatRoomInfo(info.roomAttributes.get(key));

                            if (mChatRoomInfo == null) {
                                setChatRoomInfo(roomInfo);
                            } else {
                                //房间模块中关于roomInfo的回调
                                if (chatRoomEventHandler != null) {
                                    chatRoomEventHandler.onChatRoomInfoUpdated(roomInfo);
                                }

                                //判断房主禁言
                                boolean originIsMuted = mChatRoomInfo.isIs_mute_msg();
                                boolean nowIsMuted = roomInfo.isIs_mute_msg();
                                if (nowIsMuted != originIsMuted) {
                                    if (chatRoomEventHandler != null) {
                                        chatRoomEventHandler.onMuteAllMessage(nowIsMuted);
                                    }
                                }
                                mChatRoomInfo = roomInfo;
                            }
                            messageManager.setChatRoomInfo(mChatRoomInfo);
                        } else {
                            //删除房间属性
                            //目的是为了应用层进行操作，监听房间属性
                            if (chatRoomEventHandler != null) {
                                chatRoomEventHandler.onChatRoomInfoUpdated(null);
                            }
                        }
                    }
                }
            }


            @Override
            public void onRoomAttributesBatchUpdated(ZIM zim, ArrayList<ZIMRoomAttributesUpdateInfo> infos, String roomID) {
                voiceSeatManager.onVoiceSeatSwitched(infos);
            }

            @Override
            public void onConnectionStateChanged(ZIM zim, ZIMConnectionState state, ZIMConnectionEvent event, JSONObject extendedData) {
                final ZIMChatRoomEventHandler chatRoomEventHandler = zimChatRoomEventHandler;
                if (chatRoomEventHandler != null)
                    chatRoomEventHandler.onConnectionStateChanged(ZIMChatRoomState.getZIMChatRoomState(state.value()), ZIMChatRoomEvent.getZIMChatRoomEvent(event.value()), extendedData);
            }

        };

    }

    public void setChatRoomInfo(ZIMChatRoomInfo mChatRoomInfo) {
        this.mChatRoomInfo = mChatRoomInfo;
    }

    /**
     * 在这里具体实现房间内的相应方法
     */
    public void init(Long appID, String appSign, boolean isTestEnv, Application application) {
        zegoExpressEngine = ZegoExpressEngine.createEngine(appID, appSign, isTestEnv, ZegoScenario.LIVE, application, null);
        zim = ZIM.create(appID, application);
        messageManager.setZim(zim);//设置zim传递过去
        voiceSeatManager.setZim(zim);
        voiceSeatManager.setZegoExpressEngine(zegoExpressEngine);
        this.application = application;

        //用于整理逻辑进行自定义的callback处理
        initZIMEventHandler();
        zim.setEventHandler(zimEventHandler);
    }

    //login,还是需要一个自己定义的回调方法
    public void login(ZIMChatRoomUserInfo userInfo, String token,
                      final OnLoginCallback onLoginCallback) {
        mUser = new ZIMChatRoomUser();
        mUser.setUserID(userInfo.getUserID());
        mUser.setUserName(userInfo.getUserName());
        messageManager.setUser(mUser);
        voiceSeatManager.setUser(mUser);
        ZIMUserInfo zimUserInfo = new ZIMUserInfo();
        zimUserInfo.userID = userInfo.getUserID();
        zimUserInfo.userName = userInfo.getUserName();
        zim.login(zimUserInfo, token, new ZIMLoggedInCallback() {
            @Override
            public void onLoggedIn(ZIMError errorInfo) {
                ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                    zimChatRoomErrorCode = SUCCESS;
                } else {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                }
                if (onLoginCallback != null) {
                    onLoginCallback.onConnectionState(zimChatRoomErrorCode);
                }
            }
        });
    }

    //  房主创建房间
    public void createChatRoom(String roomID,
                               String roomName,
                               final String rtcToken,
                               final CreateChatRoomCallback createChatRoomCallback) {
        mUser.setUserRole(ZIMChatRoomUserRole.OWNER);
        mChatRoomInfo = new ZIMChatRoomInfo();
        mChatRoomInfo.setRoom_id(roomID);
        mChatRoomInfo.setRoom_Name(roomName);
        mChatRoomInfo.setAuthor(mUser.getUserID());
        mChatRoomInfo.setSeat_count(8);
        mChatRoomInfo.setIs_mute_msg(false);

        messageManager.setChatRoomInfo(mChatRoomInfo);//设置给Message模块中房间属性
        voiceSeatManager.setRoomID(roomID);
        ZIMRoomInfo zimRoomInfo = new ZIMRoomInfo();
        zimRoomInfo.roomID = roomID;
        zimRoomInfo.roomName = roomName;

        HashMap<String, String> roomAttributes = new HashMap<String, String>();
        roomAttributes.put("room_info", GsonChanger.getInstance().getJsonOfZIMChatRoomInfo(mChatRoomInfo));
        ZIMRoomAdvancedConfig config = new ZIMRoomAdvancedConfig();
        config.roomAttributes = roomAttributes;

        zim.createRoom(zimRoomInfo, config, new ZIMRoomCreatedCallback() {
            @Override
            public void onRoomCreated(ZIMRoomFullInfo roomInfo, ZIMError errorInfo) {
                if (errorInfo.code == ZIMErrorCode.SUCCESS) {

                    createChatRoomCallback.onCreateRoomState(SUCCESS);

                    voiceSeatManager.setupRTCModule(rtcToken, new SetupRTCModuleCallback() {
                        @Override
                        public void onConnectionState(ZIMChatRoomErrorCode error) {

                        }
                    });

                } else {
                    if (errorInfo.code == ZIMErrorCode.THE_ROOM_ALREADY_EXISTS) {
                        createChatRoomCallback.onCreateRoomState(ZIMChatRoomErrorCode.ROOM_EXISTED);
                    } else {
                        createChatRoomCallback.onCreateRoomState(ZIMChatRoomErrorCode.ERROR);
                    }
                }
            }
        });

    }

    //观众加入语聊房
    public void joinChatRoom(String roomID, final String rtcToken, final JoinChatRoomCallback joinChatRoomCallback) {
        mUser.setUserRole(ZIMChatRoomUserRole.VISITOR);//设置为观众
        voiceSeatManager.setRoomID(roomID);
        mChatRoomInfo = new ZIMChatRoomInfo();
        mChatRoomInfo.setRoom_id(roomID);
        mChatRoomInfo.setAuthor(mUser.getUserID());
        mChatRoomInfo.setSeat_count(8);
        mChatRoomInfo.setIs_mute_msg(false);
        messageManager.setChatRoomInfo(mChatRoomInfo);
        zim.joinRoom(roomID, new ZIMRoomJoinedCallback() {
            @Override
            public void onRoomJoined(ZIMRoomFullInfo roomInfo, ZIMError errorInfo) {
                //ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                    if (joinChatRoomCallback != null) {
                        joinChatRoomCallback.onConnectionState(ZIMChatRoomErrorCode.SUCCESS);
                    }

                    //启动 RTC 模块
                    voiceSeatManager.setupRTCModule(rtcToken, new SetupRTCModuleCallback() {
                        @Override
                        public void onConnectionState(ZIMChatRoomErrorCode error) {

                        }
                    });
                } else if (errorInfo.code.equals(ZIMErrorCode.ROOM_DOES_NOT_EXIST)) {

                    if (joinChatRoomCallback != null) {
                        joinChatRoomCallback.onConnectionState(ZIMChatRoomErrorCode.ROOM_NOT_FOUND);
                    }

                } else {
                    if (joinChatRoomCallback != null) {
                        joinChatRoomCallback.onConnectionState(ZIMChatRoomErrorCode.ERROR);
                    }
                }
            }
        });
    }

    //离开语聊房
    public void leaveChatRoom(String roomID, final OnLeaveCallback onLeaveCallback) {

        voiceSeatManager.releaseRTCModule();
        zim.leaveRoom(roomID, new ZIMRoomLeftCallback() {
            @Override
            public void onRoomLeft(String roomID, ZIMError errorInfo) {
                ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                    zimChatRoomErrorCode = SUCCESS;
                } else {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                }
                if (onLeaveCallback != null) {
                    onLeaveCallback.onConnectionState(zimChatRoomErrorCode);
                }
            }
        });
        voiceSeatManager.releaseVideoSeats();
    }

    //更新RTC业务Token
    public void renewRTCToken(String token) {
        zegoExpressEngine.renewToken(mChatRoomInfo.getRoom_id(), token);
    }

    //更新ZIM业务Token
    public void renewZIMToken(String token) {
        zim.renewToken(token, null);
    }

    //摧毁语聊房
    public void unInit() {
        zim.destroy();
        //RTC 的 摧毁
        ZegoExpressEngine.destroyEngine(null);
    }

    /**
     * 用户查询的功能
     */
    //分页拉取整个语聊房的用户
    public void queryRoomMember(String roomID, ZIMChatRoomQueryMemberConfig config, final OnQueryRoomMemberCallback onQueryRoomMember) {
        ZIMRoomMemberQueryConfig zimConfig = new ZIMRoomMemberQueryConfig();
        zimConfig.count = config.getCount();
        zimConfig.nextFlag = config.getNextFlag();
        zim.queryRoomMemberList(roomID, zimConfig, new ZIMRoomMemberQueriedCallback() {
            @Override
            public void onRoomMemberQueried(String roomID, ArrayList<ZIMUserInfo> memberList, String nextFlag, ZIMError errorInfo) {
                ArrayList<ZIMChatRoomUser> chatRoomUsers = getChatRoomUsers(memberList);//转化
                ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                    zimChatRoomErrorCode = SUCCESS;
                } else {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                }
                if (onQueryRoomMember != null) {
                    onQueryRoomMember.onQueryRoomMember(chatRoomUsers, nextFlag, zimChatRoomErrorCode);
                }
            }
        });
    }

    /*
         Util
    */
    //进行转化，将原SDK中的User对象进行封装变成我们自定义的对象
    private ArrayList<ZIMChatRoomUser> getChatRoomUsers(ArrayList<ZIMUserInfo> memberList) {
        ArrayList<ZIMChatRoomUser> chatRoomUsers = new ArrayList<>();
        for (ZIMUserInfo userInfo : memberList) {
            //存储ChatRoomUser
            ZIMChatRoomUser chatRoomUser = new ZIMChatRoomUser();
            chatRoomUser.setUserID(userInfo.userID);
            chatRoomUser.setUserName(userInfo.userName);
            //读取整体的房间属性然后获取一个房间属性对象在进行上述的操作
            if (userInfo.userID.equals(mChatRoomInfo.getAuthor())) {
                chatRoomUser.setUserRole(ZIMChatRoomUserRole.OWNER);
            } else {
                chatRoomUser.setUserRole(ZIMChatRoomUserRole.VISITOR);
            }
            chatRoomUsers.add(chatRoomUser);
        }
        return chatRoomUsers;
    }

    public void queryChatRoomInfo(String roomID, final OnQueryChatRoomInfoCallback onQueryRoomMember) {
        final ZIMChatRoomInfo[] zimChatRoomInfo = new ZIMChatRoomInfo[1];
        zim.queryRoomAllAttributes(roomID, new ZIMRoomAttributesQueriedCallback() {
            @Override
            public void onRoomAttributesQueried(String roomID, HashMap<String, String> roomAttributes, ZIMError errorInfo) {
                String room_info_string = roomAttributes.get("room_info");
                zimChatRoomInfo[0] = GsonChanger.getInstance().getZIMChatRoomInfo(room_info_string);
                onQueryRoomMember.onQueryRoomMember(zimChatRoomInfo[0]);
            }
        });
    }


    /**
     * 消息模块，在这里实现消息通信的相关操作；
     */
    /*
        消息模块的功能
     */
    //发送房间消息
    public void sendRoomMessage(String message, SendRoomMessageCallback sendRoomMessageCallback) {
        messageManager.sendRoomMessage(message, sendRoomMessageCallback);
    }

    //发送礼物消息
    public void sendGiftMessage(int giftType, ArrayList<String> userIDs, SendGiftMessageCallback
            sendGiftMessageCallback) {
        messageManager.sendGiftMessage(giftType, userIDs, sendGiftMessageCallback);
    }

    //发送邀请 判断是否发送成功
    public void sendInvitation(String toUserID, SendInvitationStatusCallback
            sendInvitationStatusCallback) {
        messageManager.sendInvitation(toUserID, sendInvitationStatusCallback);
    }

    //回应邀请
    public void respondInvitation(ZIMChatRoomInvitationStatus
                                          status, SendInvitationStatusCallback sendInvitationStatusCallback) {
        messageManager.respondInvitation(status, sendInvitationStatusCallback);
    }

    //全员禁言
    public void muteAllMessage(boolean isMuted, MuteAllMessageCallback muteAllMessageCallback) {
        messageManager.muteAllMessage(isMuted, muteAllMessageCallback);
    }

    /**
     * 麦位管理
     */
    public void kickUserFromSeat(String userID, KickUserToSeatCallback kickUserToSeat) {
        voiceSeatManager.kickUserFromSeat(userID, kickUserToSeat);
    }

    public void lockSeat(boolean isLocked, int seatIndex, LockSeatCallback lockSeat) {
        voiceSeatManager.lockSeat(isLocked, seatIndex, lockSeat);
    }

    public void muteSeat(boolean isMuted, MuteSeatCallback muteSeat) {
        voiceSeatManager.muteSeat(isMuted, muteSeat);
    }

    public void enterSeat(int seatIndex, EnterSeatCallback enterSeatCallback) {
        voiceSeatManager.enterSeat(seatIndex, false, enterSeatCallback);
    }

    public void leaveSeat(LeaveSeatCallback leaveSeatCallback) {
        voiceSeatManager.leaveSeat(leaveSeatCallback);
    }

    public void switchSeat(int toSeatIndex, SwitchSeatCallback switchSeatCallback) {
        voiceSeatManager.switchSeat(toSeatIndex, switchSeatCallback);
    }

    public void muteSpeaker(boolean isMuted) {
        voiceSeatManager.getAudioProfileManager().muteSpeaker(isMuted);
    }

    public void logout() {
        if (zim != null) {
            zim.logout();
        }
    }


}
