package im.zego.zimchatroom.internal;

import im.zego.zim.entity.ZIMMessageSendConfig;
import im.zego.zim.enums.ZIMMessagePriority;
import im.zego.zimchatroom.callback.MuteAllMessageCallback;
import im.zego.zimchatroom.callback.SendGiftMessageCallback;
import im.zego.zimchatroom.callback.SendInvitationStatusCallback;
import im.zego.zimchatroom.callback.SendRoomMessageCallback;
import im.zego.zimchatroom.emus.ZIMChatRoomErrorCode;
import im.zego.zimchatroom.emus.ZIMChatRoomInvitationStatus;
import im.zego.zimchatroom.emus.ZIMChatRoomMessageAction;
import im.zego.zimchatroom.entity.ZIMChatRoomGift;
import im.zego.zimchatroom.entity.ZIMChatRoomGiftBroadcast;
import im.zego.zimchatroom.internal.entity.ZIMChatRoomInfo;
import im.zego.zimchatroom.entity.ZIMChatRoomInvitation;
import im.zego.zimchatroom.entity.ZIMChatRoomInvitationRespond;
import im.zego.zimchatroom.entity.ZIMChatRoomMessage;
import im.zego.zimchatroom.entity.ZIMChatRoomText;
import im.zego.zimchatroom.entity.ZIMChatRoomUser;
import im.zego.zimchatroom.util.GsonChanger;

import java.util.ArrayList;
import java.util.HashMap;

import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMMessageSentCallback;
import im.zego.zim.callback.ZIMRoomAttributesOperatedCallback;
import im.zego.zim.entity.ZIMError;
import im.zego.zim.entity.ZIMMessage;
import im.zego.zim.entity.ZIMRoomAttributesSetConfig;
import im.zego.zim.entity.ZIMTextMessage;
import im.zego.zim.enums.ZIMErrorCode;

/**
 * 消息管理模块，在这里实现消息的相关操作，邀请消息等等操作，
 */
public class ZIMMessageManager {
    ZIM zim;
    ZIMChatRoomUser user;
    String fromInvitationUserID;
    //传递属性
    ZIMChatRoomInfo chatRoomInfo;

    public ZIMMessageManager() {

    }

    public ZIM getZim() {
        return zim;
    }

    public void setZim(ZIM zim) {
        this.zim = zim;
    }


    public ZIMChatRoomUser getUser() {
        return user;
    }

    public void setUser(ZIMChatRoomUser user) {
        this.user = user;
    }

    public String getFromInvitationUserID() {
        return fromInvitationUserID;
    }

    public void setFromInvitationUserID(String fromInvitationUserID) {
        this.fromInvitationUserID = fromInvitationUserID;
    }

    public ZIMChatRoomInfo getChatRoomInfo() {
        return chatRoomInfo;
    }

    public void setChatRoomInfo(ZIMChatRoomInfo chatRoomInfo) {
        this.chatRoomInfo = chatRoomInfo;

    }

    /**
     * 在这里具体实现消息的相应方法
     */
    //发送房间消息
    public void sendRoomMessage(String message, final SendRoomMessageCallback sendRoomMessageCallback) {
        //进行文本处理
        ZIMChatRoomText textMessage = new ZIMChatRoomText();
        textMessage.setContent(message);
        textMessage.setFromUserID(user.getUserID());
        ZIMChatRoomMessage zimChatRoomMessage = new ZIMChatRoomMessage(ZIMChatRoomMessageAction.TEXT, textMessage);
        String realMessage = GsonChanger.getInstance().getJsonOfZIMChatRoomMessage(zimChatRoomMessage);
        ZIMTextMessage zimTextMessage = new ZIMTextMessage();
        zimTextMessage.message = realMessage;
        if (!chatRoomInfo.isIs_mute_msg() || (chatRoomInfo.getAuthor() != null && chatRoomInfo.getAuthor().equals(user.getUserID()))) {
            ZIMMessageSendConfig config = new ZIMMessageSendConfig();
            config.priority = ZIMMessagePriority.HIGH;
            zim.sendRoomMessage(zimTextMessage, chatRoomInfo.getRoom_id(), config, new ZIMMessageSentCallback() {
                @Override
                public void onMessageSent(ZIMMessage message, ZIMError errorInfo) {
                    ZIMChatRoomErrorCode zimChatRoomErrorCode;
                    if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    } else {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                    }
                    sendRoomMessageCallback.onSendRoomMessage(zimChatRoomErrorCode);
                }
            });
        } else {
            sendRoomMessageCallback.onSendRoomMessage(ZIMChatRoomErrorCode.ERROR);
        }
    }

    //发送礼物消息
    public void sendGiftMessage(int giftType, final ArrayList<String> userIDs, final SendGiftMessageCallback sendGiftMessageCallback) {
        ZIMChatRoomGift gift = new ZIMChatRoomGift();
        gift.setGiftType(giftType);
        ZIMChatRoomMessage zimChatRoomMessage = new ZIMChatRoomMessage(ZIMChatRoomMessageAction.GIFT, gift);
        //gosn转化
        String message = GsonChanger.getInstance().getJsonOfZIMChatRoomMessage(zimChatRoomMessage);
        ZIMTextMessage zimTextMessage = new ZIMTextMessage();
        zimTextMessage.message = message;
        final ArrayList<String> failToUserIDList = new ArrayList<>();
        //先各个单发再群发
        for (final String toUserID : userIDs) {
            ZIMMessageSendConfig config = new ZIMMessageSendConfig();
            config.priority = ZIMMessagePriority.MEDIUM;
            zim.sendPeerMessage(zimTextMessage, toUserID, config, new ZIMMessageSentCallback() {
                @Override
                public void onMessageSent(ZIMMessage message, ZIMError errorInfo) {
                    if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {

                    } else {
                        failToUserIDList.add(toUserID);
                        userIDs.remove(toUserID);
                    }
                }
            });
        }
        //礼物单独发送还得广播出去
        //配置GiftBroadcast
        ZIMChatRoomGiftBroadcast giftBroadcast = new ZIMChatRoomGiftBroadcast();
        giftBroadcast.setGiftType(giftType);
        giftBroadcast.setToUserIDList(userIDs);
        ZIMChatRoomMessage chatRoomGiftBroadcastMessage = new ZIMChatRoomMessage(ZIMChatRoomMessageAction.GIFT_BROADCAST, giftBroadcast);
        //转化为gson
        String giftBroadcastMessage = GsonChanger.getInstance().getJsonOfZIMChatRoomMessage(chatRoomGiftBroadcastMessage);
        //组装
        ZIMTextMessage zimGiftBroadcastTextMessage = new ZIMTextMessage();
        zimGiftBroadcastTextMessage.message = giftBroadcastMessage;
        ZIMMessageSendConfig config = new ZIMMessageSendConfig();
        config.priority = ZIMMessagePriority.MEDIUM;
        zim.sendRoomMessage(zimGiftBroadcastTextMessage, chatRoomInfo.getRoom_id(), config, new ZIMMessageSentCallback() {
            @Override
            public void onMessageSent(ZIMMessage message, ZIMError errorInfo) {
                ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    //一个回调
                } else {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                }
                sendGiftMessageCallback.onSendGiftMessage(zimChatRoomErrorCode, failToUserIDList);
            }
        });
    }

    //发送邀请 判断是否发送成功
    public void sendInvitation(String toUserID, final SendInvitationStatusCallback sendInvitationStatusCallback) {
        //构建邀请信息
        ZIMChatRoomInvitation invitation = new ZIMChatRoomInvitation();
        invitation.setFromUserID(user.getUserID());
        ZIMChatRoomMessage invitationMessage = new ZIMChatRoomMessage(ZIMChatRoomMessageAction.INVITATION, invitation);
        String message = GsonChanger.getInstance().getJsonOfZIMChatRoomMessage(invitationMessage);
        ZIMTextMessage zimMessage = new ZIMTextMessage();
        zimMessage.message = message;
        ZIMMessageSendConfig config = new ZIMMessageSendConfig();
        config.priority = ZIMMessagePriority.HIGH;
        zim.sendPeerMessage(zimMessage, toUserID, config, new ZIMMessageSentCallback() {
            @Override
            public void onMessageSent(ZIMMessage message, ZIMError errorInfo) {
                ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    //一个回调
                } else {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                }
                sendInvitationStatusCallback.sendInvitationStatus(zimChatRoomErrorCode);
            }
        });

    }

    //回应邀请
    public void respondInvitation(ZIMChatRoomInvitationStatus status, final SendInvitationStatusCallback sendInvitationStatusCallback) {
        //构建邀请反馈的信息
        ZIMChatRoomInvitationRespond invitationRespond = new ZIMChatRoomInvitationRespond();
        invitationRespond.setStatus(status);
        ZIMChatRoomMessage invitationMessage = new ZIMChatRoomMessage(ZIMChatRoomMessageAction.INVITATION_RESPOND, invitationRespond);
        String message = GsonChanger.getInstance().getJsonOfZIMChatRoomMessage(invitationMessage);
        ZIMTextMessage zimMessage = new ZIMTextMessage();
        zimMessage.message = message;
        ZIMMessageSendConfig config = new ZIMMessageSendConfig();
        config.priority = ZIMMessagePriority.HIGH;
        zim.sendPeerMessage(zimMessage, fromInvitationUserID, config, new ZIMMessageSentCallback() {
            @Override
            public void onMessageSent(ZIMMessage message, ZIMError errorInfo) {
                ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    //一个回调
                } else {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                }
                sendInvitationStatusCallback.sendInvitationStatus(zimChatRoomErrorCode);
            }
        });
    }

    //全员禁言
    public void muteAllMessage(boolean isMuted, final MuteAllMessageCallback muteAllMessageCallback) {
        HashMap<String, String> roomConfig = new HashMap<String, String>();
        chatRoomInfo.setIs_mute_msg(isMuted);
        //先对象转string再转字节
        String string_config = GsonChanger.getInstance().getJsonOfZIMChatRoomInfo(chatRoomInfo);
        roomConfig.put("room_info", string_config);
        //设置好isForce为true
        ZIMRoomAttributesSetConfig setConfig = new ZIMRoomAttributesSetConfig();
        setConfig.isForce = true;
        setConfig.isDeleteAfterOwnerLeft = true;
        zim.setRoomAttributes(roomConfig, chatRoomInfo.getRoom_id(), setConfig, new ZIMRoomAttributesOperatedCallback() {
            @Override
            public void onRoomAttributesOperated(String roomID, ArrayList<String> errorKeys, ZIMError errorInfo) {
                ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    //一个回调
                } else {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                }
                muteAllMessageCallback.onMuteAllMessage(zimChatRoomErrorCode);
            }
        });
    }

}
