package im.zego.zimchatroom.internal;


import static im.zego.zimchatroom.emus.ZIMChatRoomErrorCode.NOT_IN_SEAT;
import static im.zego.zimchatroom.emus.ZIMChatRoomErrorCode.SUCCESS;
import static im.zego.zimchatroom.emus.ZIMChatRoomSeatEvent.ENTERED;
import static im.zego.zimchatroom.emus.ZIMChatRoomSeatEvent.LEFT;

import android.util.Log;

import im.zego.zimchatroom.callback.EnterSeatCallback;
import im.zego.zimchatroom.callback.KickUserToSeatCallback;
import im.zego.zimchatroom.callback.LeaveSeatCallback;
import im.zego.zimchatroom.callback.LockSeatCallback;
import im.zego.zimchatroom.callback.MuteSeatCallback;
import im.zego.zimchatroom.callback.SetupRTCModuleCallback;
import im.zego.zimchatroom.callback.SwitchSeatCallback;
import im.zego.zimchatroom.callback.ZIMChatRoomEventHandler;
import im.zego.zimchatroom.emus.ZIMChatRoomErrorCode;
import im.zego.zimchatroom.emus.ZIMChatRoomSeatEvent;
import im.zego.zimchatroom.emus.ZIMChatRoomVoiceStatus;
import im.zego.zimchatroom.entity.ZIMVoiceSeat;
import im.zego.zimchatroom.entity.ZIMChatRoomSeatAttribution;
import im.zego.zimchatroom.entity.ZIMChatRoomUser;
import im.zego.zimchatroom.entity.ZIMVoiceSeatUpdateInfo;
import im.zego.zimchatroom.util.GsonChanger;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.callback.IZegoEventHandler;
import im.zego.zegoexpress.constants.ZegoPlayerState;
import im.zego.zegoexpress.constants.ZegoPublisherState;
import im.zego.zegoexpress.constants.ZegoRoomState;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoRoomConfig;
import im.zego.zegoexpress.entity.ZegoUser;
import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMRoomAttributesBatchOperatedCallback;
import im.zego.zim.callback.ZIMRoomAttributesOperatedCallback;
import im.zego.zim.entity.ZIMError;
import im.zego.zim.entity.ZIMRoomAttributesBatchOperationConfig;
import im.zego.zim.entity.ZIMRoomAttributesDeleteConfig;
import im.zego.zim.entity.ZIMRoomAttributesSetConfig;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;
import im.zego.zim.enums.ZIMErrorCode;
import im.zego.zim.enums.ZIMRoomAttributesUpdateAction;

/**
 * 麦位管理模块，在这里实现麦位的相关操作，邀请上麦等等操作，
 */
public class ZIMVoiceSeatManager {
    ZIM zim;
    ZegoExpressEngine zegoExpressEngine;
    IZegoEventHandler expressHandler;
    SetupRTCModuleCallback setupRTCModuleCallback;

    ZIMChatRoomUser user;//该用户的
    String mRoomID;
    ZIMAudioProfileManager audioProfileManager;//子管理类，如AudioProfileManager
    //存放的相关属性
    ArrayList<ZIMVoiceSeat> mVideoSeats;// 这些更动的东西要监听的，不断根据现在房间属性实时监听

    public ZIMVoiceSeatManager() {
        audioProfileManager = new ZIMAudioProfileManager();
        mVideoSeats = initAllSeatListInRoom();
    }

    private void initExpressHandler() {
        expressHandler = new IZegoEventHandler() {
            //监听Express是否过期
            @Override
            public void onRoomTokenWillExpire(String roomID, int remainTimeInSecond) {
                super.onRoomTokenWillExpire(roomID, remainTimeInSecond);
                final ZIMChatRoomEventHandler chatRoomEventHandler = ZIMChatRoomManager.getInstance().getEventHandler();
                if (chatRoomEventHandler != null) {
                    chatRoomEventHandler.onRTCTokenWillExpire(roomID, remainTimeInSecond);
                }
            }

            @Override
            public void onPublisherQualityUpdate(String streamID, ZegoPublishStreamQuality quality) {
                super.onPublisherQualityUpdate(streamID, quality);
            }

            @Override
            public void onPublisherStateUpdate(String streamID, ZegoPublisherState state, int errorCode, JSONObject extendedData) {
                super.onPublisherStateUpdate(streamID, state, errorCode, extendedData);
                if (state == ZegoPublisherState.NO_PUBLISH && errorCode != 0) {
                    final ZIMChatRoomEventHandler chatRoomEventHandler = ZIMChatRoomManager.getInstance().getEventHandler();
                    if (chatRoomEventHandler != null) {
                        chatRoomEventHandler.onRTCModuleError(errorCode);
                    }
                }
            }

            @Override
            public void onPlayerStateUpdate(String streamID, ZegoPlayerState state, int errorCode, JSONObject extendedData) {
                super.onPlayerStateUpdate(streamID, state, errorCode, extendedData);
                if (state == ZegoPlayerState.NO_PLAY && errorCode != 0) {
                    final ZIMChatRoomEventHandler chatRoomEventHandler = ZIMChatRoomManager.getInstance().getEventHandler();
                    if (chatRoomEventHandler != null) {
                        chatRoomEventHandler.onRTCModuleError(errorCode);
                    }
                }
            }

            @Override
            public void onPlayerQualityUpdate(String streamID, ZegoPlayStreamQuality quality) {
                super.onPlayerQualityUpdate(streamID, quality);
            }

            @Override
            public void onRoomStateUpdate(String roomID, ZegoRoomState state, int errorCode, JSONObject extendedData) {

                ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (state != ZegoRoomState.CONNECTING) {
                    if (errorCode == 0 && state == ZegoRoomState.CONNECTED) {
                        zimChatRoomErrorCode = SUCCESS;

                    } else {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                    }

                    final SetupRTCModuleCallback callback = setupRTCModuleCallback;
                    if (callback != null) {
                        callback.onConnectionState(zimChatRoomErrorCode);
                    }
                }

                if (state == ZegoRoomState.DISCONNECTED && errorCode != 0) {
                    final ZIMChatRoomEventHandler chatRoomEventHandler = ZIMChatRoomManager.getInstance().getEventHandler();
                    if (chatRoomEventHandler != null) {
                        chatRoomEventHandler.onRTCModuleError(errorCode);
                    }
                }
            }

            @Override
            public void onCapturedSoundLevelUpdate(float soundlevel) {
                final ZIMChatRoomEventHandler chatRoomEventHandler = ZIMChatRoomManager.getInstance().getEventHandler();
                super.onCapturedSoundLevelUpdate(soundlevel);
                if (chatRoomEventHandler != null) {
                    chatRoomEventHandler.OnLocalUserSoundLevelUpdated(soundlevel);
                }
            }

            //声浪监听回调
            @Override
            public void onRemoteSoundLevelUpdate(HashMap<String, Float> soundLevels) {
                final ZIMChatRoomEventHandler chatRoomEventHandler = ZIMChatRoomManager.getInstance().getEventHandler();
                super.onRemoteSoundLevelUpdate(soundLevels);
                if (chatRoomEventHandler != null) {
                    chatRoomEventHandler.OnRemoteUserSoundLevelUpdated(soundLevels);
                }
            }
        };
    }

    public ZIMAudioProfileManager getAudioProfileManager() {
        return audioProfileManager;
    }

    //RTC所有的接口不向外抛
    //启动RTC模块
    public void setupRTCModule(String rtcToken, final SetupRTCModuleCallback setupRTCModuleCallback) {
        this.setupRTCModuleCallback = setupRTCModuleCallback;

        ZegoUser zegoUser = new ZegoUser(user.getUserID(), user.getUserName());
        ZegoRoomConfig roomConfig = new ZegoRoomConfig();
        roomConfig.token = rtcToken;
        zegoExpressEngine.loginRoom(mRoomID, zegoUser, roomConfig);
        zegoExpressEngine.startSoundLevelMonitor();
    }

    //释放RTC资源 不外抛，自用
    public void releaseRTCModule() {
        zegoExpressEngine.stopSoundLevelMonitor();
        zegoExpressEngine.stopPublishingStream();
        zegoExpressEngine.logoutRoom(mRoomID);
    }

    private static ArrayList<ZIMVoiceSeat> initAllSeatListInRoom() {
        ArrayList<ZIMVoiceSeat> allSeats = new ArrayList<>();
        //初始化所有麦位信息，数组下标即为麦位index
        for (int i = 0; i <= 7; i++) {
            ZIMVoiceSeat voiceSeat = new ZIMVoiceSeat();
            voiceSeat.getAttribution().setIndex(i);
            voiceSeat.setStatus(ZIMChatRoomVoiceStatus.UNUSED);
            allSeats.add(voiceSeat);
        }
        return allSeats;
    }

    public ZIM getZim() {
        return zim;
    }

    public void setZim(ZIM zim) {
        this.zim = zim;
    }

    public void setZegoExpressEngine(ZegoExpressEngine zegoExpressEngine) {
        this.zegoExpressEngine = zegoExpressEngine;
        audioProfileManager.setExpressEngine(zegoExpressEngine);
        initExpressHandler();
        this.zegoExpressEngine.setEventHandler(expressHandler);
    }

    public ZIMChatRoomUser getUser() {
        return user;
    }

    public void setUser(ZIMChatRoomUser user) {
        this.user = user;
    }

    public ZIMVoiceSeat getVoiceSeat(int index) {
        return mVideoSeats.get(index);
    }

    public void setRoomID(String roomID) {
        this.mRoomID = roomID;
    }

    public void onVoiceSeatAdded(ZIMVoiceSeat voiceSeat) {
        int index = voiceSeat.getAttribution().getIndex();
        mVideoSeats.set(index, voiceSeat);

        if (voiceSeat.getAttribution().getUser_id().equals(this.user.getUserID())) {
            zegoExpressEngine.startPublishingStream(this.user.getUserID());
        } else {
            zegoExpressEngine.startPlayingStream(voiceSeat.getAttribution().getUser_id(), null);
        }

        final ZIMChatRoomEventHandler handler = ZIMChatRoomManager.getInstance().getEventHandler();
        if (handler != null) {
            ZIMVoiceSeatUpdateInfo info = new ZIMVoiceSeatUpdateInfo(voiceSeat, ZIMChatRoomSeatEvent.ENTERED);
            ArrayList<ZIMVoiceSeatUpdateInfo> infoList = new ArrayList<>();
            infoList.add(info);
            handler.onChatRoomVoiceSeatUpdated(infoList);
        }
    }

    public void onVoiceSeatMuted(int seatIndex, boolean isMuted) {
        ZIMVoiceSeat seat = mVideoSeats.get(seatIndex);
        seat.getAttribution().setIs_muted(isMuted);
        mVideoSeats.set(seatIndex, seat);

        final ZIMChatRoomEventHandler handler = ZIMChatRoomManager.getInstance().getEventHandler();
        if (handler != null) {
            ZIMVoiceSeatUpdateInfo info = new ZIMVoiceSeatUpdateInfo(seat, isMuted ? ZIMChatRoomSeatEvent.MUTED : ZIMChatRoomSeatEvent.UNMUTED);
            ArrayList<ZIMVoiceSeatUpdateInfo> infoList = new ArrayList<>();
            infoList.add(info);
            handler.onChatRoomVoiceSeatUpdated(infoList);
        }

    }

    public void onVoiceSeatRemoved(int seatIndex) {
        // 替换成空的 seat

        ZIMVoiceSeat zimVoiceSeat = mVideoSeats.get(seatIndex);
        zimVoiceSeat.setStatus(ZIMChatRoomVoiceStatus.UNUSED);
        if (zimVoiceSeat.getAttribution().getUser_id().equals(this.user.getUserID())) {
            zegoExpressEngine.stopPublishingStream();
        } else {
            zegoExpressEngine.stopPlayingStream(zimVoiceSeat.getAttribution().getUser_id());
        }

        final ZIMChatRoomEventHandler handler = ZIMChatRoomManager.getInstance().getEventHandler();
        if (handler != null) {
            ZIMVoiceSeat seat = new ZIMVoiceSeat();
            seat.setStatus(zimVoiceSeat.getStatus());
            ZIMChatRoomSeatAttribution attribution = new ZIMChatRoomSeatAttribution();
            attribution.setIs_locked(zimVoiceSeat.getAttribution().isIs_locked());
            attribution.setUser_id(zimVoiceSeat.getAttribution().getUser_id());
            attribution.setIndex(zimVoiceSeat.getAttribution().getIndex());
            attribution.setIs_muted(zimVoiceSeat.getAttribution().isIs_muted());
            attribution.setExtras(zimVoiceSeat.getAttribution().getExtras());
            seat.setAttribution(attribution);
            ZIMVoiceSeatUpdateInfo info = new ZIMVoiceSeatUpdateInfo(seat, ZIMChatRoomSeatEvent.LEFT);
            ArrayList<ZIMVoiceSeatUpdateInfo> infoList = new ArrayList<>();
            infoList.add(info);
            zimVoiceSeat.getAttribution().setUser_id("");
            handler.onChatRoomVoiceSeatUpdated(infoList);
        }


        mVideoSeats.set(seatIndex, zimVoiceSeat);
    }

    public void onVoiceSeatSwitched(ArrayList<ZIMRoomAttributesUpdateInfo> infoList) {
        final ZIMChatRoomEventHandler chatRoomEventHandler = ZIMChatRoomManager.getInstance().getEventHandler();
        ArrayList<ZIMVoiceSeatUpdateInfo> seatUpdateInfos = new ArrayList<>();

        for (ZIMRoomAttributesUpdateInfo info : infoList) {
            Set<String> keys = info.roomAttributes.keySet();
            if (info.action == ZIMRoomAttributesUpdateAction.SET) {
                //设置的就是切麦后位置，删除是切麦前的位置
                for (String key : keys) {
                    ZIMChatRoomSeatAttribution seatAttribution = GsonChanger.getInstance().getZIMChatRoomSeatAttribution(info.roomAttributes.get(key));
                    ZIMVoiceSeat voiceSeat = new ZIMVoiceSeat();
                    voiceSeat.setAttribution(seatAttribution);
                    voiceSeat.setStatus(ZIMChatRoomVoiceStatus.USED);
                    mVideoSeats.set(seatAttribution.getIndex(), voiceSeat);
                    ZIMVoiceSeatUpdateInfo updateInfo = new ZIMVoiceSeatUpdateInfo(voiceSeat, ENTERED);
                    seatUpdateInfos.add(updateInfo);//添加到回调数据中
                }
            } else {
                for (String key : keys) {
                    String idxStr = key.split("_")[1];
                    int index = Integer.valueOf(idxStr).intValue();
                    ZIMVoiceSeat voiceSeat = new ZIMVoiceSeat();
                    voiceSeat.getAttribution().setIndex(index);
                    voiceSeat.setStatus(ZIMChatRoomVoiceStatus.UNUSED);
                    mVideoSeats.set(index, voiceSeat);
                    ZIMVoiceSeatUpdateInfo updateInfo = new ZIMVoiceSeatUpdateInfo(voiceSeat, LEFT);
                    seatUpdateInfos.add(updateInfo);//添加到回调数据中
                }
            }
        }

        if (chatRoomEventHandler != null) {
            chatRoomEventHandler.onChatRoomVoiceSeatUpdated(seatUpdateInfos);
        }
    }


    public void onVoiceSeatLocked(int seatIndex, boolean isLocked) {

        ZIMVoiceSeat seat = mVideoSeats.get(seatIndex);
        seat.getAttribution().setIs_locked(isLocked);
        seat.setStatus(isLocked ? ZIMChatRoomVoiceStatus.LOCKED : ZIMChatRoomVoiceStatus.UNUSED);
        mVideoSeats.set(seatIndex, seat);

        final ZIMChatRoomEventHandler handler = ZIMChatRoomManager.getInstance().getEventHandler();
        if (handler != null) {
            ZIMVoiceSeatUpdateInfo info = new ZIMVoiceSeatUpdateInfo(seat, isLocked ? ZIMChatRoomSeatEvent.LOCKED : ZIMChatRoomSeatEvent.UNLOCKED);
            ArrayList<ZIMVoiceSeatUpdateInfo> infoList = new ArrayList<>();
            infoList.add(info);
            handler.onChatRoomVoiceSeatUpdated(infoList);
        }
    }

    /**
     * 在这里具体实现麦位操作的相应方法
     */
    /*
    房主端的操作
     */
    //踢观众下麦
    public void kickUserFromSeat(String userID, final KickUserToSeatCallback kickUserToSeatCallback) {
        //删除房间属性关于该麦位的信息
        ZIMVoiceSeat chatRoomSeat = getZIMChatRoomSeatByUserID(userID);
        if (chatRoomSeat != null) {
            int index = chatRoomSeat.getAttribution().getIndex();
            ArrayList<String> keys = new ArrayList<>();
            keys.add("seat_" + index);
            ZIMRoomAttributesDeleteConfig deleteConfig = new ZIMRoomAttributesDeleteConfig();
            deleteConfig.isForce = true;//踢人改为true
            zim.deleteRoomAttributes(keys, mRoomID, deleteConfig, new ZIMRoomAttributesOperatedCallback() {
                @Override
                public void onRoomAttributesOperated(String roomID, ArrayList<String> errorKeys, ZIMError errorInfo) {
                    ZIMChatRoomErrorCode zimChatRoomErrorCode;
                    if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    } else {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                    }
                    kickUserToSeatCallback.kickUserToSeat(zimChatRoomErrorCode);
                }
            });
        } else {
            kickUserToSeatCallback.kickUserToSeat(NOT_IN_SEAT);
        }

    }

    //锁麦和解锁麦位
    public void lockSeat(boolean isLocked, int seatIndex, final LockSeatCallback lockSeatCallback) {
        if (isLocked) {
            HashMap<String, String> roomAttributes = new HashMap<String, String>();
            //添加相应的麦位信息
            ZIMChatRoomSeatAttribution seatAttributes = new ZIMChatRoomSeatAttribution();
            seatAttributes.setIndex(seatIndex);
            seatAttributes.setUser_id("");
            seatAttributes.setIs_muted(false);
            seatAttributes.setIs_locked(true);

            ZIMRoomAttributesSetConfig setConfig = new ZIMRoomAttributesSetConfig();
            setConfig.isForce = false;
            setConfig.isDeleteAfterOwnerLeft = true;

            //设置房间属性
            roomAttributes.put("seat_" + seatIndex, GsonChanger.getInstance().getJsonOfZIMChatRoomSeat(seatAttributes));

            //更改房间信息
            zim.setRoomAttributes(roomAttributes, mRoomID, setConfig, new ZIMRoomAttributesOperatedCallback() {
                @Override
                public void onRoomAttributesOperated(String roomID, ArrayList<String> errorKeys, ZIMError errorInfo) {
                    ZIMChatRoomErrorCode zimChatRoomErrorCode;
                    if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    } else {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                    }
                    lockSeatCallback.lockSeat(zimChatRoomErrorCode);
                }
            });
        } else {
            String seatIndexKey = "seat_" + seatIndex;
            ArrayList<String> keyList = new ArrayList<>();
            keyList.add(seatIndexKey);
            ZIMRoomAttributesDeleteConfig config = new ZIMRoomAttributesDeleteConfig();
            config.isForce = true;
            zim.deleteRoomAttributes(keyList, mRoomID, config, new ZIMRoomAttributesOperatedCallback() {
                @Override
                public void onRoomAttributesOperated(String roomID, ArrayList<String> errorKeys, ZIMError errorInfo) {
                    ZIMChatRoomErrorCode zimChatRoomErrorCode;
                    if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    } else {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                    }
                    lockSeatCallback.lockSeat(zimChatRoomErrorCode);
                }
            });
        }

    }

    //静音（麦克风）该用户,这个是房主和观众都可用的（仅限在上麦）
    public void muteSeat(boolean isMuted, final MuteSeatCallback muteSeatCallback) {

        ZIMVoiceSeat mySeat = findMySeat();
        if (mySeat != null) {
            // 获取该用户的麦位信息，然后再进行修改
            audioProfileManager.muteMicrophone(isMuted);
            // 修改麦位属性通知大家修改了该麦位的麦克风
            mySeat.getAttribution().setIs_muted(isMuted);
            HashMap<String, String> seatAttributes = new HashMap<String, String>();
            seatAttributes.put("seat_" + mySeat.getAttribution().getIndex(),
                    GsonChanger.getInstance().getJsonOfZIMChatRoomSeat(mySeat.getAttribution()));
            ZIMRoomAttributesSetConfig setConfig = new ZIMRoomAttributesSetConfig();
            setConfig.isDeleteAfterOwnerLeft = true;
            setConfig.isForce = false;
            zim.setRoomAttributes(seatAttributes, mRoomID, setConfig, new ZIMRoomAttributesOperatedCallback() {
                @Override
                public void onRoomAttributesOperated(String roomID, ArrayList<String> errorKeys, ZIMError errorInfo) {
                    ZIMChatRoomErrorCode zimChatRoomErrorCode;
                    if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    } else {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                    }
                    muteSeatCallback.muteSeat(zimChatRoomErrorCode);
                }
            });
        } else {
            muteSeatCallback.muteSeat(ZIMChatRoomErrorCode.NOT_IN_SEAT);
        }
    }


    /*
    所有操作
     */
    //上麦
    public void enterSeat(final int seatIndex, boolean isSwitchSeat, final EnterSeatCallback enterSeatCallback) {

        if (!isInSeat(seatIndex) && (isSwitchSeat || isInSeat(user.getUserID()) == -1)) {
            //如果麦位没人用才可以上麦
            HashMap<String, String> seatAttributes = new HashMap<String, String>();
            //添加相应的麦位信息
            ZIMChatRoomSeatAttribution roomAttributes = new ZIMChatRoomSeatAttribution();
            roomAttributes.setIndex(seatIndex);
            roomAttributes.setUser_id(user.getUserID());
            roomAttributes.setIs_muted(false);
            roomAttributes.setIs_locked(false);
            mVideoSeats.get(seatIndex).getAttribution().setUser_id(user.getUserID());
            ZIMRoomAttributesSetConfig setConfig = new ZIMRoomAttributesSetConfig();
            setConfig.isForce = false;
            setConfig.isDeleteAfterOwnerLeft = true;
            //设置房间属性
            seatAttributes.put("seat_" + seatIndex, GsonChanger.getInstance().getJsonOfZIMChatRoomSeat(roomAttributes));
            audioProfileManager.muteMicrophone(false);
            zim.setRoomAttributes(seatAttributes, mRoomID, setConfig, new ZIMRoomAttributesOperatedCallback() {
                @Override
                public void onRoomAttributesOperated(String roomID, ArrayList<String> errorKeys, ZIMError errorInfo) {
                    ZIMChatRoomErrorCode zimChatRoomErrorCode;
                    if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    } else {
                        mVideoSeats.get(seatIndex).getAttribution().setUser_id("");
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.SET_SEAT_INFO_FAILED;
                    }
                    enterSeatCallback.enterSeat(zimChatRoomErrorCode);
                }
            });
        } else {
            enterSeatCallback.enterSeat(ZIMChatRoomErrorCode.SEAT_EXISTED);//有人在该麦位上了
        }
    }


    //下麦
    public void leaveSeat(final LeaveSeatCallback leaveSeatCallback) {
        //删除房间属性关于该麦位的信息
        int index = isInSeat(user.getUserID());
        if (index != -1) {
            ZIMVoiceSeat mySeat = findMySeat();
            // 离开麦位后重置静音状态
            if (mySeat != null) {
                audioProfileManager.muteMicrophone(false);
                mySeat.getAttribution().setIs_muted(false);
            }
            ArrayList<String> keys = new ArrayList<>();
            keys.add("seat_" + index);
            ZIMRoomAttributesDeleteConfig deleteConfig = new ZIMRoomAttributesDeleteConfig();
            deleteConfig.isForce = false;
            zim.deleteRoomAttributes(keys, mRoomID, deleteConfig, new ZIMRoomAttributesOperatedCallback() {
                @Override
                public void onRoomAttributesOperated(String roomID, ArrayList<String> errorKeys, ZIMError errorInfo) {
                    ZIMChatRoomErrorCode zimChatRoomErrorCode;
                    if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    } else {
                        zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                    }
                    leaveSeatCallback.leaveSeat(zimChatRoomErrorCode);
                }
            });
        } else {
            leaveSeatCallback.leaveSeat(ZIMChatRoomErrorCode.ERROR);
        }


    }

    boolean isSwitch = false;

    //跳麦
    public void switchSeat(int toSeatIndex, final SwitchSeatCallback switchSeatCallback) {
        //先删原来的房间属性，再增加一个房间属性，采用批处理的方式进行处理
        final ZIMVoiceSeat mySeat = findMySeat();
        if (mySeat == null) {
            switchSeatCallback.switchSeat(NOT_IN_SEAT);
            return;
        }
        if (isSwitch) {
            return;
        }
        isSwitch = true;

        ZIMRoomAttributesBatchOperationConfig operationConfig = new ZIMRoomAttributesBatchOperationConfig();
        operationConfig.isForce = false;
        operationConfig.isDeleteAfterOwnerLeft = true;

        //批处理开始
        zim.beginRoomAttributesBatchOperation(mRoomID, operationConfig);
        //上麦即可，即先set后delete的逻辑关系
        final boolean isMuted;
        if (mySeat.getAttribution().isIs_muted()) isMuted = true;
        else isMuted = false;

        enterSeat(toSeatIndex, true, new EnterSeatCallback() {
            @Override
            public void enterSeat(ZIMChatRoomErrorCode error) {
                if (error == SUCCESS) {

                }
            }
        });

        //删除
        ZIMRoomAttributesDeleteConfig deleteConfig = new ZIMRoomAttributesDeleteConfig();
        deleteConfig.isForce = true;
        ArrayList<String> keys = new ArrayList<>();
        final int originIndex = mySeat.getAttribution().getIndex();
        keys.add("seat_" + originIndex);
        zim.deleteRoomAttributes(keys, mRoomID, deleteConfig, new ZIMRoomAttributesOperatedCallback() {
            @Override
            public void onRoomAttributesOperated(String roomID, ArrayList<String> errorKeys, ZIMError errorInfo) {

            }
        });

        // 结束批处理
        zim.endRoomAttributesBatchOperation(mRoomID, new ZIMRoomAttributesBatchOperatedCallback() {
            @Override
            public void onRoomAttributesBatchOperated(String roomID, ZIMError errorInfo) {
                ZIMChatRoomErrorCode zimChatRoomErrorCode;
                if (errorInfo.code.equals(ZIMErrorCode.SUCCESS)) {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.SUCCESS;
                    if (isMuted) {
                        muteSeat(isMuted, new MuteSeatCallback() {
                            @Override
                            public void muteSeat(ZIMChatRoomErrorCode error) {

                            }
                        });
                    }
                } else {
                    zimChatRoomErrorCode = ZIMChatRoomErrorCode.ERROR;
                    mVideoSeats.get(originIndex).getAttribution().setUser_id("");
                }
                isSwitch = false;
                switchSeatCallback.switchSeat(zimChatRoomErrorCode);
            }
        });

    }

    public boolean isVoiceSeatUsed(int index) {
        return mVideoSeats.get(index).getStatus() == ZIMChatRoomVoiceStatus.USED;
    }

    public ZIMVoiceSeat findMySeat() {
        for (int i = 0; i < mVideoSeats.size(); i++) {
            ZIMVoiceSeat seat = mVideoSeats.get(i);
            if (user.getUserID().equals(seat.getAttribution().getUser_id())) {
                return seat;
            }
        }
        return null;
    }

    //通过userID获取一个对象
    public ZIMVoiceSeat getZIMChatRoomSeatByUserID(String userID) {
        for (ZIMVoiceSeat roomSeat : mVideoSeats) {
            if (roomSeat.getAttribution().getUser_id() != null && roomSeat.getAttribution().getUser_id().equals(userID)) {
                return roomSeat;
            }
            continue;
        }
        return null;
    }

    //获取是否在麦位上的回调
    private boolean isInSeat(int seatIndex) {
        for (ZIMVoiceSeat roomSeat : mVideoSeats) {
            if (roomSeat.getStatus().equals(ZIMChatRoomVoiceStatus.USED) && roomSeat.getAttribution().getIndex() == seatIndex) {
                return true;
            }
        }
        return false;
    }

    //获取是否在麦位上的回调
    private int isInSeat(String userID) {
        for (ZIMVoiceSeat roomSeat : mVideoSeats) {
            if (userID.equals(roomSeat.getAttribution().getUser_id())) {
                return roomSeat.getAttribution().getIndex();
            }
        }
        return -1;
    }

    public void releaseVideoSeats() {
        mVideoSeats = initAllSeatListInRoom();
    }

}
