package im.zego.zimchatroom.internal.impl;

import android.app.Application;

import im.zego.zimchatroom.ZIMChatRoom;
import im.zego.zimchatroom.callback.CreateChatRoomCallback;
import im.zego.zimchatroom.callback.EnterSeatCallback;
import im.zego.zimchatroom.callback.JoinChatRoomCallback;
import im.zego.zimchatroom.callback.KickUserToSeatCallback;
import im.zego.zimchatroom.callback.LeaveSeatCallback;
import im.zego.zimchatroom.callback.LockSeatCallback;
import im.zego.zimchatroom.callback.MuteAllMessageCallback;
import im.zego.zimchatroom.callback.MuteSeatCallback;
import im.zego.zimchatroom.callback.OnLeaveCallback;
import im.zego.zimchatroom.callback.OnLoginCallback;
import im.zego.zimchatroom.callback.OnQueryChatRoomInfoCallback;
import im.zego.zimchatroom.callback.OnQueryRoomMemberCallback;
import im.zego.zimchatroom.callback.SendGiftMessageCallback;
import im.zego.zimchatroom.callback.SendInvitationStatusCallback;
import im.zego.zimchatroom.callback.SendRoomMessageCallback;
import im.zego.zimchatroom.callback.SwitchSeatCallback;
import im.zego.zimchatroom.callback.ZIMChatRoomEventHandler;
import im.zego.zimchatroom.emus.ZIMChatRoomInvitationStatus;
import im.zego.zimchatroom.entity.ZIMChatRoomQueryMemberConfig;
import im.zego.zimchatroom.entity.ZIMChatRoomUserInfo;
import im.zego.zimchatroom.internal.ZIMChatRoomManager;

import java.util.ArrayList;


public class ZIMChatRoomImpl extends ZIMChatRoom {
    private static ZIMChatRoom instance;

    public ZIMChatRoomImpl() {

    }

    //单例实现
    public static ZIMChatRoom getInstance() {
        if (instance == null) {
            instance = new ZIMChatRoomImpl();
        }
        return instance;
    }

    @Override
    public void setZIMChatRoomEventHandler(ZIMChatRoomEventHandler chatRoomEventHandler) {
        ZIMChatRoomManager.getInstance().setEventHandler(chatRoomEventHandler);
    }

    /**
     * 房间模块
     */
    @Override
    public void init(Long appID, String appSign, Application application) {
        ZIMChatRoomManager.getInstance().init(appID, appSign, false, application);
    }

    @Override
    public void login(ZIMChatRoomUserInfo userInfo, String token, OnLoginCallback callback) {
        ZIMChatRoomManager.getInstance().login(userInfo, token, callback);
    }

    @Override
    public void createChatRoom(String roomID, String roomName, String rtcToken, CreateChatRoomCallback createChatRoomCallback) {
        ZIMChatRoomManager.getInstance().createChatRoom(roomID, roomName, rtcToken, createChatRoomCallback);
    }


    @Override
    public void joinChatRoom(String roomID, String rtcToken, JoinChatRoomCallback joinChatRoomCallback) {
        ZIMChatRoomManager.getInstance().joinChatRoom(roomID, rtcToken, joinChatRoomCallback);
    }

    @Override
    public void leaveChatRoom(String roomID, OnLeaveCallback onLeaveCallback) {
        ZIMChatRoomManager.getInstance().leaveChatRoom(roomID, onLeaveCallback);
    }

    @Override
    public void renewRTCToken(String token) {
        ZIMChatRoomManager.getInstance().renewRTCToken(token);
    }

    @Override
    public void renewZIMToken(String token) {
        ZIMChatRoomManager.getInstance().renewZIMToken(token);
    }

    @Override
    public void unInit() {
        ZIMChatRoomManager.getInstance().unInit();
    }

    @Override
    public void queryRoomMember(String roomID, ZIMChatRoomQueryMemberConfig config, OnQueryRoomMemberCallback onQueryRoomMember) {
        ZIMChatRoomManager.getInstance().queryRoomMember(roomID, config, onQueryRoomMember);
    }

    @Override
    public void queryChatRoomInfo(String roomID, OnQueryChatRoomInfoCallback onQueryRoomMember) {
        ZIMChatRoomManager.getInstance().queryChatRoomInfo(roomID, onQueryRoomMember);
    }


    /**
        Message模块
     */
    @Override
    public void sendRoomMessage(String message, SendRoomMessageCallback sendRoomMessageCallback) {
        ZIMChatRoomManager.getInstance().sendRoomMessage(message, sendRoomMessageCallback);
    }

    @Override
    public void sendGiftMessage(int giftType, ArrayList<String> userIDs, SendGiftMessageCallback sendGiftMessageCallback) {
        ZIMChatRoomManager.getInstance().sendGiftMessage(giftType, userIDs, sendGiftMessageCallback);
    }

    @Override
    public void sendInvitation(String toUserID, SendInvitationStatusCallback sendInvitationStatusCallback) {
        ZIMChatRoomManager.getInstance().sendInvitation(toUserID, sendInvitationStatusCallback);
    }

    @Override
    public void respondInvitation(ZIMChatRoomInvitationStatus status, SendInvitationStatusCallback sendInvitationStatusCallback) {
        ZIMChatRoomManager.getInstance().respondInvitation(status, sendInvitationStatusCallback);
    }

    @Override
    public void muteAllMessage(boolean isMuted, MuteAllMessageCallback muteAllMessageCallback) {
        ZIMChatRoomManager.getInstance().muteAllMessage(isMuted, muteAllMessageCallback);
    }


    /**
     *
     *麦位管理
     */
    @Override
    public void kickUserToSeat(String userID, KickUserToSeatCallback kickUserToSeat) {
        ZIMChatRoomManager.getInstance().kickUserFromSeat(userID, kickUserToSeat);
    }

    @Override
    public void lockSeat(boolean isLocked, int seatIndex, LockSeatCallback lockSeat) {
        ZIMChatRoomManager.getInstance().lockSeat(isLocked, seatIndex, lockSeat);
    }

    @Override
    public void muteSeat(boolean isMuted, MuteSeatCallback muteSeat) {
        ZIMChatRoomManager.getInstance().muteSeat(isMuted, muteSeat);
    }

    @Override
    public void enterSeat(int seatIndex, EnterSeatCallback enterSeatCallback) {
        ZIMChatRoomManager.getInstance().enterSeat(seatIndex, enterSeatCallback);
    }

    @Override
    public void leaveSeat(LeaveSeatCallback leaveSeatCallback) {
        ZIMChatRoomManager.getInstance().leaveSeat(leaveSeatCallback);
    }

    @Override
    public void switchSeat(int toSeatIndex, SwitchSeatCallback switchSeatCallback) {
        ZIMChatRoomManager.getInstance().switchSeat(toSeatIndex, switchSeatCallback);
    }

    @Override
    public void logout() {
        ZIMChatRoomManager.getInstance().logout();
    }

    @Override
    public void muteSpeaker(boolean isMuted) {
        ZIMChatRoomManager.getInstance().muteSpeaker(isMuted);
    }

}
